@@ -1,4 +1,4 @@
-#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+#     Copyright 2025, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
 
 
 """ Utils for file and directory operations.
@@ -30,9 +30,11 @@ from nuitka.__past__ import (  # pylint: disable=redefined-builtin
     basestring,
     unicode,
 )
+from nuitka.Errors import NuitkaFilenameError
 from nuitka.PythonVersions import python_version
 from nuitka.Tracing import general, my_print, options_logger, queryUser
 
+from .Hashing import Hash
 from .Importing import importFromInlineCopy
 from .ThreadedExecutor import RLock, getThreadIdent
 from .Utils import (
@@ -440,7 +442,7 @@ def getFileList(
             fullname = os.path.join(root, filename)
 
             if normalize:
-                fullname = os.path.normpath(fullname)
+                fullname = getNormalizedPath(fullname)
 
             result.append(fullname)
 
@@ -555,7 +557,7 @@ def listExeFilesFromDirectory(path, prefix=None, suffixes=None):
     # On Windows, we check exe suffixes, on other platforms we shell all filenames,
     # matching the prefix, but they have to the executable bit set.
     if not suffixes and isWin32OrPosixWindows():
-        suffixes = "exe", "bin"
+        suffixes = ("exe", "bin")
 
     if suffixes:
         pattern_list = [prefix + "*." + suffix for suffix in suffixes]
@@ -687,9 +689,20 @@ def getFilenameExtension(path):
 
 def changeFilenameExtension(path, extension):
     """Change the filename extension."""
+
+    is_legal, illegal_reason = isLegalPath(path)
+    if not is_legal:
+        raise NuitkaFilenameError(illegal_reason)
+
     return os.path.splitext(path)[0] + extension
 
 
+def switchFilenameExtension(path, old_extension, new_extension):
+    """Switch the filename extension specified to another one."""
+    assert path.endswith(old_extension)
+    return path[: -len(old_extension)] + new_extension
+
+
 def hasFilenameExtension(path, extensions):
     """Has a filename one of the given extensions.
 
@@ -774,13 +787,31 @@ def resetDirectory(path, logger, ignore_errors, extra_recommendation):
 
 
 @contextmanager
-def withTemporaryFile(suffix="", mode="w", delete=True, temp_path=None):
+def withTemporaryFile(prefix="", suffix="", mode="w", delete=True, temp_path=None):
+    """Provide a temporary file opened and potentially deleted."""
     with tempfile.NamedTemporaryFile(
-        suffix=suffix, mode=mode, delete=delete, dir=temp_path
+        prefix=prefix, suffix=suffix, mode=mode, delete=delete, dir=temp_path
     ) as temp_file:
         yield temp_file
 
 
+@contextmanager
+def withTemporaryFilename(prefix="", suffix="", temp_path=None):
+    """Provide a temporary filename."""
+    with tempfile.NamedTemporaryFile(
+        prefix=prefix,
+        suffix=suffix,
+        mode="wb",
+        delete=False,
+        dir=temp_path,
+    ) as temp_file:
+        filename = temp_file.name
+        temp_file.close()
+        deleteFile(filename, must_exist=True)
+
+        yield filename
+
+
 def getFileContentByLine(filename, mode="r", encoding=None, errors=None):
     # We read the whole, to keep lock times minimal. We only deal with small
     # files like this normally.
@@ -827,6 +858,14 @@ def getFileFirstLine(filename, mode="r", encoding=None):
 
 
 def openTextFile(filename, mode, encoding=None, errors=None):
+    # Do not attempt to create files with what we consider
+    # illegal filenames.
+    if "w" in mode:
+        is_legal, illegal_reason = isLegalPath(filename)
+        if not is_legal:
+            raise NuitkaFilenameError(illegal_reason)
+
+    # Doesn't exist anymore for Python3.7 or later.
     if python_version >= 0x370:
         mode = mode.replace("U", "")
 
@@ -890,7 +929,7 @@ def changeTextFileContents(filename, contents, encoding=None, compare_only=False
         or getFileContents(filename, encoding=encoding) != contents
     ):
         if not compare_only:
-            putTextFileContents(filename, contents)
+            putTextFileContents(filename, contents, encoding=encoding)
 
         return True
     else:
@@ -981,6 +1020,17 @@ def copyTree(source_path, dest_path):
         return copy_tree(source_path, dest_path)
 
 
+def resolveSymlink(path):
+    """Resolve a symlink, to a relative path."""
+    link_source_abs = os.path.abspath(path)
+    link_target_abs = os.path.abspath(
+        os.path.join(os.path.dirname(path), os.readlink(path))
+    )
+    link_target_rel = relpath(link_target_abs, os.path.dirname(link_source_abs))
+
+    return link_target_rel
+
+
 def copyFileWithPermissions(source_path, dest_path, dist_dir):
     """Improved version of shutil.copy2 for putting things to dist folder
 
@@ -989,12 +1039,7 @@ def copyFileWithPermissions(source_path, dest_path, dist_dir):
     """
 
     if os.path.islink(source_path) and not isWin32Windows():
-        link_source_abs = os.path.abspath(source_path)
-        link_target_abs = os.path.abspath(
-            os.path.join(os.path.dirname(source_path), os.readlink(source_path))
-        )
-
-        link_target_rel = relpath(link_target_abs, os.path.dirname(link_source_abs))
+        link_target_rel = resolveSymlink(source_path)
 
         if isFilenameBelowPath(
             path=dist_dir,
@@ -1079,7 +1124,7 @@ def isFilenameBelowPath(path, filename, consider_short=True):
         if getWindowsDrive(path) != getWindowsDrive(filename):
             return False
 
-    result = os.path.relpath(filename, path).split(os.path.sep)[0] != ".."
+    result = os.path.relpath(filename, path).split(os.path.sep, 1)[0] != ".."
 
     if not result and consider_short:
         if os.path.exists(filename) and os.path.exists(path):
@@ -1090,13 +1135,16 @@ def isFilenameBelowPath(path, filename, consider_short=True):
                 if getWindowsDrive(path) != getWindowsDrive(filename):
                     return False
 
-            result = os.path.relpath(filename, path).split(os.path.sep)[0] != ".."
+            result = os.path.relpath(filename, path).split(os.path.sep, 1)[0] != ".."
 
     return result
 
 
 def isFilenameSameAsOrBelowPath(path, filename):
     """Is a filename inside of a given directory path or the same path as that directory."""
+    if type(path) in (tuple, list):
+        return any(isFilenameSameAsOrBelowPath(path=p, filename=filename) for p in path)
+
     return isFilenameBelowPath(path, filename) or areSamePaths(path, filename)
 
 
@@ -1168,7 +1216,7 @@ def getWindowsLongPathName(filename):
     while True:
         output_buf = ctypes.create_unicode_buffer(output_buf_size)
         needed = GetLongPathNameW(
-            os.path.abspath(filename), output_buf, output_buf_size
+            getNormalizedPath(os.path.abspath(filename)), output_buf, output_buf_size
         )
 
         if needed == 0:
@@ -1229,6 +1277,14 @@ def getExternalUsePath(filename, only_dirname=False):
 _report_path_cache = {}
 
 
+def getReportSourceReference(source_ref):
+    """Convert a source ref into a path suitable for user output."""
+    return "%s:%s" % (
+        getReportPath(source_ref.getFilename()),
+        source_ref.getLineNumber(),
+    )
+
+
 def getReportPath(filename, prefixes=()):
     """Convert filename into a path suitable for reporting, avoiding home directory paths."""
     key = filename, tuple(prefixes)
@@ -1239,6 +1295,15 @@ def getReportPath(filename, prefixes=()):
     return _report_path_cache[key]
 
 
+def isNonLocalPath(path):
+    """Tell if a path is potentially outside of current directory.
+
+    This is not reliable and mainly for reporting purposes to identify paths
+    work looking to abbreviate.
+    """
+    return path.startswith("..") or os.path.isabs(path)
+
+
 def _getReportPath(filename, prefixes):
     if os.path.isabs(os.path.expanduser(filename)):
         prefixes = list(prefixes)
@@ -1463,6 +1528,9 @@ def isLegalPath(path):
     illegal_suffixes = "/\\"
     illegal_chars = "\0"
 
+    if path == "":
+        return False, "is empty"
+
     if isWin32Windows():
         illegal_chars += r'*"/<>:|?'
 
@@ -1472,7 +1540,10 @@ def isLegalPath(path):
     if isMacOS():
         illegal_chars += ":"
 
-    for c in path:
+    for pos, c in enumerate(path):
+        if c == ":" and pos == 1 and isWin32Windows():
+            continue
+
         if c in illegal_chars:
             return False, "contains illegal character %r" % c
 
@@ -1481,13 +1552,20 @@ def isLegalPath(path):
             return False, "contains illegal suffix %r" % illegal_suffix
 
         for part in path.split(os.path.sep):
-            if part == ".":
+            if part in (".", ".."):
                 continue
 
             if part.endswith(illegal_suffix):
                 return False, "contains illegal suffix %r in path part %r" % (
-                    part,
                     illegal_suffix,
+                    part,
+                )
+
+            part_length = len(part)
+            if part_length > 255:
+                return False, "contains too long (%d) name part %r" % (
+                    part_length,
+                    part,
                 )
 
     return True, None
@@ -1515,9 +1593,22 @@ def getNormalizedPath(path):
     if isWin32Windows():
         path = path.replace("/", "\\")
 
+    if isMacOS():
+        path = os.path.expanduser(path)
+
     return path
 
 
+def getFileContentsHash(filename, as_string=True, line_filter=None):
+    result = Hash()
+    result.updateFromFile(filename=filename, line_filter=line_filter)
+
+    if as_string:
+        return result.asHexDigest()
+    else:
+        return result.asDigest()
+
+
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
