@@ -1,4 +1,4 @@
-#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+#     Copyright 2025, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
 
 
 """ Utility module.
@@ -237,6 +237,11 @@ def isMacOS():
     return getOS() == "Darwin"
 
 
+def isAIX():
+    """The AIX platform."""
+    return getOS() == "AIX"
+
+
 def hasMacOSIntelSupport():
     """macOS with either Intel hardware or Rosetta being installed."""
     return isMacOS() and (
@@ -288,7 +293,14 @@ def getArchitecture():
         else:
             return "x86"
     else:
-        return os.uname()[4]
+        result = os.uname()[4]
+
+        if isAIX():
+            # Translate known values to what -X would expect.
+            if result == "00C63E504B00":
+                return "64"
+
+        return result
 
 
 def getCPUCoreCount():
@@ -313,32 +325,22 @@ def getCPUCoreCount():
     return cpu_count
 
 
-def encodeNonAscii(var_name):
-    """Encode variable name that is potentially not ASCII to ASCII only.
-
-    For Python3, unicode identifiers can be used, but these are not
-    possible in C, so we need to replace them.
-    """
-    if str is bytes:
-        return var_name
-    else:
-        # Using a escaping here, because that makes it safe in terms of not
-        # to occur in the encoding escape sequence for unicode use.
-        # spell-checker: ignore xmlcharrefreplace
-        var_name = var_name.replace("$$", "$_$")
-
-        var_name = var_name.encode("ascii", "xmlcharrefreplace")
-        var_name = var_name.decode("ascii")
-
-        return var_name.replace("&#", "$$").replace(";", "")
-
-
 def hasOnefileSupportedOS():
-    return getOS() in ("Linux", "Windows", "Darwin", "FreeBSD")
+    """Is onefile supported on this OS. Please help to add more."""
+    # List the exceptions where standalone works, but onefile does not.
+    return hasStandaloneSupportedOS() and getOS not in ("OpenBSD",)
 
 
 def hasStandaloneSupportedOS():
-    return getOS() in ("Linux", "Windows", "Darwin", "FreeBSD", "OpenBSD")
+    """Is standalone supported on this OS. Please help to add more."""
+    return getOS() in (
+        "Linux",
+        "Windows",
+        "Darwin",
+        "FreeBSD",
+        "OpenBSD",
+        "AIX",
+    )
 
 
 def getUserName():
@@ -392,6 +394,15 @@ def withNoWarning():
         yield
 
 
+@contextmanager
+def withNoExceptions():
+    """Only let RuntimeError and the like escape."""
+    try:
+        yield
+    except Exception:  # pylint: disable=broad-exception-caught
+        pass
+
+
 def decoratorRetries(
     logger,
     purpose,
@@ -482,6 +493,121 @@ def getLaunchingNuitkaProcessEnvironmentValue(environment_variable_name):
     return getLaunchingNuitkaProcessEnvironmentValue(environment_variable_name)
 
 
+def isRPathUsingPlatform():
+    """Does the OS have an rpath or equivalent."""
+    return isElfUsingPlatform() or isMacOS()
+
+
+def isElfUsingPlatform():
+    """Does the OS use the ELF file format."""
+    return not isWin32OrPosixWindows() and not isMacOS() and not isAIX()
+
+
+def isCoffUsingPlatform():
+    """Does the OS use the COFF file format."""
+    return isAIX()
+
+
+_msvc_redist_path = None
+
+
+def _getMSVCRedistPath(logger):
+    # The detection is return driven with many cases to look at
+    # pylint: disable=too-many-return-statements
+
+    # TODO: We could try and export the vswhere information from what Scons
+    # found out, to avoid the (then duplicated) vswhere call entirely.
+
+    # The program name is from the installer, spell-checker: ignore vswhere
+    vswhere_path = None
+    for candidate in ("ProgramFiles(x86)", "ProgramFiles"):
+        program_files_dir = os.getenv(candidate)
+
+        if program_files_dir is not None:
+            candidate = os.path.join(
+                program_files_dir,
+                "Microsoft Visual Studio",
+                "Installer",
+                "vswhere.exe",
+            )
+
+            if os.path.exists(candidate):
+                vswhere_path = candidate
+                break
+
+    if vswhere_path is None:
+        return None
+
+    # Cyclic imports
+    from .Execution import executeToolChecked
+
+    command = (
+        vswhere_path,
+        "-latest",
+        "-property",
+        "installationPath",
+        "-products",
+        "*",
+        "-prerelease",
+    )
+
+    vs_path = executeToolChecked(
+        logger=logger,
+        command=command,
+        absence_message="requiring vswhere for redist discovery",
+        decoding=True,
+    ).strip()
+
+    redist_base_path = os.path.join(vs_path, "VC", "Redist", "MSVC")
+
+    if not os.path.exists(redist_base_path):
+        return None
+
+    try:
+        all_folders = [
+            d
+            for d in os.listdir(redist_base_path)
+            if os.path.isdir(os.path.join(redist_base_path, d))
+        ]
+        if not all_folders:
+            return None
+
+        version_folders = [v for v in all_folders if v[0].isdigit()]
+
+        if not version_folders:
+            return None
+
+        latest_version = sorted(
+            version_folders, key=lambda v: list(map(int, v.split(".")))
+        )[-1]
+
+    except (IOError, ValueError):
+        return None
+
+    arch_folder_map = {
+        "x86_64": "x64",
+        "arm64": "arm64",
+        "x86": "x86",
+    }
+    arch_folder = arch_folder_map.get(getArchitecture())
+    final_path = os.path.join(redist_base_path, latest_version, arch_folder)
+
+    if os.path.exists(final_path):
+        return final_path
+
+    return None
+
+
+def getMSVCRedistPath(logger):
+    global _msvc_redist_path  # singleton, pylint: disable=global-statement
+
+    if _msvc_redist_path is None:
+        if isWin32Windows():
+            _msvc_redist_path = _getMSVCRedistPath(logger=logger)
+
+    return _msvc_redist_path
+
+
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
